/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."

 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import org.apache.jmeter.samplers.*;
import org.apache.jmeter.visualizers.*;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.threads.*;
import org.apache.jmeter.control.*;
import org.apache.jmeter.reporters.*;
import org.apache.jmeter.gui.*;
import org.apache.jmeter.engine.*;
import org.apache.jmeter.util.*;

/**
 * The main startup class for JMeter.
 */
public class Driver extends JFrame implements ActionListener
{
    JMenuItem run_start, run_stop, run_clear;
    JComboBox visualizerList;
    JPanel visualizers;

    JComboBox controllerList;
    JComboBox timerList;
    JPanel controllers;
    JPanel timers;
    JDialog about;

    org.apache.jmeter.timers.Timer timer;
    Visualizer visualizer;
    VisualizerContainer vcn;
    ThreadGroupPanel tp;
    SamplerController controller;

    Properties properties;

    JMeterEngine engine;

    //LagDetector lagger = new LagDetector();

    public Driver()
    {
	this(null);
    }

    public Driver(String file) {
      super("JMeter - Java Apache Project");
	init(file);
	initGUI();
    }

    protected void init(String file) {
	this.properties = JMeterUtils.getProperties(file);
	engine = new JMeterEngine();
    }

    /**
    * Creates the MenuBar for this application.  I believe in my heart that
    * this should be defined in a file somewhere, but that is for later.
    */
    private JMenuBar makeMenuBar()
    {

	JMenuBar menuBar = new JMenuBar();

	JMenu fileMenu = new JMenu("File");
	fileMenu.setMnemonic('F');
	JMenuItem file_exit = new JMenuItem("Exit",'X');
	file_exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,KeyEvent.ALT_MASK));
	file_exit.setActionCommand("exit");
	file_exit.addActionListener(this);
	fileMenu.add(file_exit);

	JMenu runMenu = new JMenu("Run");
	runMenu.setMnemonic('R');
	run_start = new JMenuItem("Start",'S');
	run_start.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,KeyEvent.CTRL_MASK));
	run_start.addActionListener(this);
	run_start.setActionCommand("start");
	run_stop = new JMenuItem("Stop",'T');
	run_stop.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PERIOD,KeyEvent.CTRL_MASK));
	run_stop.setEnabled(false);
	run_stop.addActionListener(this);
	run_stop.setActionCommand("stop");
	run_clear = new JMenuItem("Clear",'C');
	run_clear.addActionListener(this);
	run_clear.setActionCommand("clear");
	runMenu.add(run_start);
	runMenu.add(run_stop);
	runMenu.add(run_clear);

	JMenu reportMenu = new JMenu("Report");
	reportMenu.setMnemonic('P');
	JMenuItem analyze = new JMenuItem("Analyze Data File...");
	analyze.setMnemonic('A');
	analyze.addActionListener(this);
	reportMenu.add(analyze);

	JMenu optionsMenu = new JMenu("Options");
	JMenu lafMenu = new JMenu("Look and Feel");
	UIManager.LookAndFeelInfo lafs[] = UIManager.getInstalledLookAndFeels();
	for(int i=0; i < lafs.length; ++i) {
		JMenuItem laf = new JMenuItem(lafs[i].getName());
		laf.addActionListener(this);
		laf.setActionCommand("laf:"+lafs[i].getClassName());
		lafMenu.add(laf);
	}
	optionsMenu.setMnemonic('O');
	optionsMenu.add(lafMenu);


	JMenu helpMenu = new JMenu("Help");
	helpMenu.setMnemonic('H');
	JMenuItem help_about = new JMenuItem("About",'A');
	help_about.setActionCommand("about");
	help_about.addActionListener(this);
	helpMenu.add(help_about);

	menuBar.add(fileMenu);
	menuBar.add(runMenu);
	menuBar.add(reportMenu);
	menuBar.add(optionsMenu);
	// This makes the help stick to the right side
	// I stole this from the java.sun.com tutorial page
	menuBar.add(Box.createHorizontalGlue());
	menuBar.add(helpMenu);
	return menuBar;
    }

    private JPanel makeTimerPanel(EtchedBorder etched)
    {
        JPanel timerPanel = new JPanel();
        timerPanel.setBorder(new TitledBorder(etched, "Timers"));
        timerPanel.setLayout(new BorderLayout());

        Vector t = JMeterUtils.getTimers(this.properties);
        timerList = new JComboBox(t);
        timerList.setEditable(false);
        timerList.setSelectedIndex(0);
        timerList.addActionListener(this);
        timerList.setActionCommand("timer");
        timerPanel.add(timerList, BorderLayout.NORTH);

        timers = new JPanel();
        timers.setLayout(new CardLayout());
	String startupTimer = JMeterUtils.getPropDefault("startup.timer",".");
        for (Enumeration e = t.elements(); e.hasMoreElements();) {
            JComponent timer = (JComponent) e.nextElement();
            timers.add(timer, timer.toString());
	    if ( startupTimer.equalsIgnoreCase(timer.getClass().getName()) ) {
		System.out.println("### Setting startup timer=" + startupTimer );
		timerList.setSelectedItem(timer);
	    }
        }
        timerPanel.add(timers, BorderLayout.CENTER);
	timer = (org.apache.jmeter.timers.Timer)timerList.getSelectedItem();

        return timerPanel;
    }

    private JPanel makeSetupPanel(EtchedBorder etched)
    {
        GridBagConstraints gbc = new GridBagConstraints();
        JPanel controllerPanel = new JPanel();
	controllerPanel.setBorder(new TitledBorder(etched, "Samplers"));
	controllerPanel.setLayout(new GridBagLayout());

	Vector a = JMeterUtils.getControllers(this.properties);
	controllerList = new JComboBox(a);
	controllerList.setEditable(false);
	controllerList.setSelectedIndex(0);
	controllerList.addActionListener(this);
	controllerList.setActionCommand("controller");

	controllers = new JPanel();
        controllers.setBorder(etched);
        controllers.setLayout(new CardLayout());
        String startupController =
	    JMeterUtils.getPropDefault("startup.controller",".");
        for (Enumeration e = a.elements(); e.hasMoreElements();) {
            controller =(SamplerController)e.nextElement();
            controller.setTestSamples(JMeterUtils.getTestSamples(this.properties,
                    controller.getPropName()));
            controllers.add(controller.getControlPanel(),
		    controller.toString());
	    if ( startupController.equalsIgnoreCase(controller.getClass().getName()) ) {
		System.out.println("### Setting Controller="+
				   startupController);
		controllerList.setSelectedItem(controller);
	    }
        }

        gbc.gridx = gbc.gridy = 0;
        gbc.anchor=gbc.CENTER;
        gbc.fill=gbc.NONE;
        gbc.gridwidth = gbc.gridheight = 1;
        gbc.weightx = gbc.weighty = 0;
	controllerPanel.add(controllerList, gbc.clone());

        gbc.gridheight=1;
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weightx=gbc.weighty=.9;
        gbc.fill=gbc.BOTH;
	controllerPanel.add(controllers, gbc.clone());

        gbc.gridy++;
        gbc.fill=gbc.NONE;
        gbc.anchor=gbc.CENTER;
        gbc.weightx=gbc.weighty=.1;
        controllerPanel.add(makeTimerPanel(etched),gbc.clone());

        return controllerPanel;
    }

    private JPanel makeRunPanel(EtchedBorder etched)
    {
        controller = (SamplerController)controllerList.getSelectedItem();
        tp = new ThreadGroupPanel(controller.getDefaultThreadGroups());
        for(int x = controllerList.getItemCount();x > 0;x--)
        {
          controller = (SamplerController)controllerList.getItemAt(x-1);
          tp.addThreadGroupListener(controller);
        }
	vcn = new VisualizerContainer(JMeterUtils.getVisualizers(this.properties), new String[] {"Group 1"});
	tp.addThreadGroupListener(vcn);

        JPanel runPanel = new JPanel();
        runPanel.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        gbc.gridx = gbc.gridy = 0;
        gbc.anchor = gbc.CENTER;
        gbc.fill = gbc.NONE;
        runPanel.add(tp,gbc.clone());

        gbc.fill = gbc.BOTH;
        gbc.gridy++;
        gbc.weightx = gbc.weighty = 1;
        runPanel.add(vcn.getVisualizerPanel(),gbc.clone());

        return runPanel;
    }

    protected void initGUI() {
	EtchedBorder etched = new EtchedBorder();
	/* This is too small with the use of the graphical buttons,
	 * and we cannot increase the size without killing our 640x480
	 * friends.  Ergo, I am converting the graphical buttons to menus,
	 * which should free up more space.
	 */
	setSize(530,450);
        getContentPane().setLayout(new GridBagLayout());
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {System.exit(0);}});

        GridBagConstraints gbc = new GridBagConstraints();

        gbc.gridy = gbc.gridx = 0;
        gbc.fill = gbc.HORIZONTAL;
        gbc.weightx = 1;
        gbc.weighty = 0;
	/*
	 * see my comment above re: the size issue.
	getContentPane().add(makeToolBar(),gbc.clone());

	*/
	setJMenuBar(makeMenuBar());

	JTabbedPane jtp = new JTabbedPane();
	jtp.addTab("Setup", makeSetupPanel(etched));
	jtp.addTab("Run", makeRunPanel(etched));

        gbc.fill = gbc.BOTH;
        gbc.weighty = 1;
        gbc.gridy++;
        getContentPane().add(jtp,gbc.clone());
    }

    /**
     * This understands the following ActionCommands:
     *<DL>
     *<DT>start<DD> Starts the threads in the thread groups.
     *<DT>stop<DD> Stops the threads in the thread groups.
     *<DT>clear<DD> Clears the statistics gathered so far.
     *<DT>visualizer<DD>Shows the visualizers.
     *<DT>controller<DD>Shows the controllers.
     *<DT>timer<DD>Shows the timers.
     *<DT>about<DD>Displays the About box.
     *<DT>exit<DD>Exits the program normally.
     *<DT>laf:...<DD>Sets the Look And Feel to the class named after the colon.
     *<DT>"Analyze Data File..."<DD>Starts up a FileReporter upon a file selected by a FileChooser
     *</DL>
     */
    public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if( cmd == null || cmd.length() < 1 ) {
		//FIXME: Do something smart here?
		return;
	}
	if ("start".equals(cmd)) {
	    run_start.setEnabled(false);
	    run_stop.setEnabled(true);
	    tp.setEnabled(false);
	    controllerList.setEnabled(false);
	    timerList.setEnabled(false);
	    ThreadMonitor[] monitors = tp.getMonitors();
	    for(int i=0;i<monitors.length;i++)
		engine.addSampleListener(monitors[i]);
	    engine.addSampleListener(vcn);
	    engine.setTimer(timer);
	    String[] groups = tp.getThreadGroupNames();
	    int[] thread_counts = new int[groups.length];
	    for(int i=0;i<groups.length;i++)
		thread_counts[i]=tp.getThreadCount(groups[i]);
	    engine.start(groups, thread_counts);
	} else if ("Analyze Data File...".equals(cmd)) {
		try {
			File tmp = JMeterUtils.promptToOpenFile().getSelectedFile();
			if( tmp != null ) {
				new FileReporter().init(tmp.getAbsolutePath());
			}
		} catch (Exception ex) {
			System.out.println(ex.toString());
		}
	} else if ("stop".equals(cmd)) {
	    engine.stop();
	    run_start.setEnabled(true);
	    run_stop.setEnabled(false);
	    tp.setEnabled(true);
	    controllerList.setEnabled(true);
	    timerList.setEnabled(true);
      engine.removeSampleListener(vcn);
	}
	else if (cmd.equals("clear")) {
            vcn.clear();
        }
	else if (cmd.equals("visualizer")) {
            this.visualizer = (Visualizer) visualizerList.getSelectedItem();
            ((CardLayout) visualizers.getLayout()).show(visualizers,
                    (String) this.visualizer.toString());
        }
	else if (cmd.equals("controller")) {
            controller =
		(SamplerController) controllerList.getSelectedItem();
            ((CardLayout)controllers.getLayout()).show(controllers,(String) this.controller.toString());
	    engine.setSamplerController(controller);
        }
	else if (cmd.equals("timer")) {
            this.timer = (org.apache.jmeter.timers.Timer) timerList.getSelectedItem();
            ((CardLayout) timers.getLayout()).show(timers,(String) this.timer.toString());
	    engine.setTimer(timer);
	}
	else if (cmd.equals("about")) {
            about();
        }
	else if (cmd.equals("exit")) {
	//FIXME: Should this do something better?
            System.exit(0);
        }
	else if (cmd.startsWith("laf:")) {
	    // eat off the laf:
	    String clsName = cmd.substring(4);
	    try {
		UIManager.setLookAndFeel(clsName);
		SwingUtilities.updateComponentTreeUI(this);
		this.pack();
	    } catch( Exception ex ) {
		/* what else can we do? */
		ex.printStackTrace();
	    }
	}
    }

    /**
     * Called by about button. Raises about dialog.
     */
    void about() {
        if (about == null) {
            about = new JDialog(this, "About Apache JMeter...", true);
            about.addMouseListener(new MouseAdapter() {
		    public void mouseClicked(MouseEvent e) {
			about.setVisible(false);
		    }
		});

            JLabel jmeter = new JLabel(JMeterUtils.getImage("jmeter.jpg"));

            JLabel copyright = new JLabel("Copyright (c) 1998-2000 The Java Apache Project", JLabel.CENTER);
            JLabel rights = new JLabel("All Rights Reserved.", JLabel.CENTER);
            JPanel infos = new JPanel();
            infos.setOpaque(false);
            infos.setLayout(new GridLayout(0, 1));
            infos.setBorder(new EmptyBorder(5, 5, 5, 5));
            infos.add(copyright);
            infos.add(rights);

            Container panel = about.getContentPane();
            panel.setLayout(new BorderLayout());
            panel.setBackground(Color.white);
            panel.add(jmeter, BorderLayout.NORTH);
            panel.add(infos, BorderLayout.SOUTH);
        }

        // NOTE: these lines center the about dialog in the
        // current window. Some older Swing versions have
        // a bug in getLocationOnScreen() and they may not
        // make this behave properly.
        Point p = this.getLocationOnScreen();
        Dimension d1 = this.getSize();
        Dimension d2 = about.getSize();
        about.setLocation(p.x + (d1.width - d2.width) / 2, p.y + (d1.height - d2.height) / 2);
        about.pack();
        about.setVisible(true);
    }

    public static void main(String[] args)
    {
	Driver app;
	if (args.length>0)
	    app = new Driver(args[0]);
	else
	    app = new Driver();
	app.setLocation(100,100);
	app.setVisible(true);
    }
    private boolean invokedStandalone = false;

}









