package org.apache.jmeter.config;

import java.util.*;

/**
 * Title:        Apache JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache Foundation
 * @author Michael Stover
 * @version 1.0
 */

public interface ConfigElement {

	/**
	 * Sets the name of the config element.
	 */
	void setName(String name);

	/**
	 * Gets the name of the config element.
	 */
	String getName();

	/**
	 * Gets a property of the config element.
	 */
	Object getProperty(String key);

	/**
	 * Sets a property of the config element.
	 */
	void putProperty(String key,Object value);

	/**
	 * Get the names of all valid properties for this config element.
	 */
	Collection getPropertyNames();

	/**
	 * This allows config elements to combine and give a "layered" effect.  for
	 * example, say there are two HTTPConfigElements, which have properties for
	 * domain, path, method, and parameters.  If element A has everything filled
	 * in, but null for domain, and element B is added, which has only domain filled
	 * in, then after adding B to A, A will have the domain from B.  If A already
	 * had a domain, then the correct behavior is for A to ignore the addition of
	 * element B.
	 */
	void addConfigElement(ConfigElement config);
}