package org.apache.jmeter.control;

import java.util.*;

import org.apache.jmeter.samplers.NewEntry;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.config.ConfigElement;

/*************************
 *  Title: Apache JMeter Description: Copyright: Copyright (c) 2000 Company:
 *  Apache Foundation
 *
 *@author     Michael Stover
 *@created    $Date: 2000/12/22 01:41:45 $
 *@version    1.0
 ************************/

public interface NewSamplerController
{

	/*************************
	 *  Get the next Entry to be sampled.
	 *
	 *@return                       Description of the Returned Value
	 *@exception  NoEntryException  Description of Exception
	 ************************/
	NewEntry nextEntry() throws NoEntryException;

	/*************************
	 *  Gets the full set of unique entries this controller has. The behavior of
	 *  this method is highly dependent on the controller's implementation.
	 *
	 *@return    Description of the Returned Value
	 ************************/
	Collection allEntries();

	/*************************
	 *  Add a config element to the controller
	 *
	 *@param  config  The feature to be added to the ConfigElement attribute
	 ************************/
	void addConfigElement(ConfigElement config);

	/*************************
	 *  Returns the name of this Controller
	 *
	 *@return    The Name value
	 ************************/
	String getName();

	/*************************
	 *  Sets the name of this controller.
	 *
	 *@param  name  The new Name value
	 ************************/
	void setName(String name);

	/*************************
	 *  Returns a sample result to the controller. Most controllers will likely
	 *  ignore this data, but some may change their entries depending on the
	 *  contents of the SampleResult (example: spiders).
	 *
	 *@param  result  Description of Parameter
	 ************************/
	void returnSampleResult(SampleResult result);

	/**
	 * Adds a SamplerController underneath this controller.  This controller will
	 * then get entries from these controllers.
	 */
	void addSamplerController(NewSamplerController controller);
}
