/*
 * Copyright (c) 1998-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.control;

import javax.swing.JPanel;
import org.apache.jmeter.threads.*;

/**
 * Establishes the actions any class that controls a  sampler
 * must provide.  Among these actions is starting, stopping and
 * configuring (via a control panel) the sampler.
 * This also includes getting information about the sampler,
 * like it's name and thread groups.
 */
public interface SamplerController extends ThreadGroupListener {
	 /**
	  * Tells the sampler to begin collecting samples in its own special way.
	  */
	 public void start();

	 /**
	  * Tells the sampler to cease collecting samples in its own special way.
	  */
	 public void stop();

	 /**
	  * Returns a container which knows how to set various parameters of
	  * the sampler.
	  */
	 public java.awt.Container getControlPanel();

	 /**
	  * Tells what the sampler likes to be called.
	  */
	 public String getName();

	 /**
	  * Tells what the sampler is called in the properties file
	  */
	 public String getPropName();

	 /**
	  * Sets the samplers name in the properties file
	  */
	 public void setPropName(String propName);

	 /**
	  * Tells which thread groups the sampler wants to belong to,
	  * unless told otherwise.
	  */
	 public String[] getDefaultThreadGroups();

	 /**
	  * Returns the actual sampler thread(s) that will be doing the work.
	  * @associates <{org.apache.jmeter.controllers.JMeterThread}>
	  */
	 public JMeterThread[] getSampleThreads(String threadGroup, int count);

	 /**
	  * Sets the list of TestSamples that this SamplerController will use.
	  */
	 public void setTestSamples(String[] testSamples);

}
