/*
 * Copyright (c) 1998-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 * software must display the following acknowledgment:
 * "This product includes software developed by the Java Apache
 * Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 * not be used to endorse or promote products derived from this software
 * without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 * nor may "Java Apache Project" appear in their names without
 * prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 * acknowledgment:
 * "This product includes software developed by the Java Apache
 * Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.gui;

import java.util.*;
import javax.swing.AbstractListModel;

import org.apache.jmeter.control.*;

/*************************
 *  Provides a way of organizing Test Samples in a JList.
 *
 *@author     mike
 *@created    December 8, 2000
 *@version    $Revision: 1.2 $
 *@see        TestSampleCellRenderer
 ************************/
public class TestSampleListModel extends AbstractListModel
{

	Vector data;
	Hashtable threadMap;


	/*************************
	 *  Constructor for the TestSampleListModel object
	 ************************/
	public TestSampleListModel()
	{
		super();
		data = new Vector();
		threadMap = new Hashtable();
	}


	/*************************
	 *  Constructor for the TestSampleListModel object
	 *
	 *@param  samples  Description of Parameter
	 ************************/
	public TestSampleListModel(TestSample[] samples)
	{
		this();
		for (int i = 0; i < samples.length; i++)
		{
			addTestSample(samples[i]);
		}
	}


	/*************************
	 *  Gets the TestSample attribute of the TestSampleListModel object
	 *
	 *@param  index  Description of Parameter
	 *@return        The TestSample value
	 ************************/
	public TestSample getTestSample(int index)
	{
		return (TestSample) data.elementAt(index);
	}


	/*************************
	 *  Gets the TestSamples attribute of the TestSampleListModel object
	 *
	 *@param  threadGroup  Description of Parameter
	 *@return              The TestSamples value
	 ************************/
	public TestSample[] getTestSamples(String threadGroup)
	{
		Vector v = (Vector) threadMap.get(threadGroup);
		if (v != null)
		{
			TestSample[] result = new TestSample[v.size()];
			v.copyInto(result);
			return result;
		}
		return new TestSample[0];
	}


	/*************************
	 *  Gets the TestSamples attribute of the TestSampleListModel object
	 *
	 *@return    The TestSamples value
	 ************************/
	public TestSample[] getTestSamples()
	{
		return (TestSample[]) data.toArray(new TestSample[0]);
	}


	/*************************
	 *  Gets the ElementAt attribute of the TestSampleListModel object
	 *
	 *@param  index  Description of Parameter
	 *@return        The ElementAt value
	 ************************/
	public Object getElementAt(int index)
	{
		return getTestSample(index);
	}


	/*************************
	 *  Gets the Size attribute of the TestSampleListModel object
	 *
	 *@return    The Size value
	 ************************/
	public int getSize()
	{
		return data.size();
	}


	/*************************
	 *  Adds a feature to the TestSample attribute of the TestSampleListModel
	 *  object
	 *
	 *@param  sample  The feature to be added to the TestSample attribute
	 ************************/
	public void addTestSample(TestSample sample)
	{
		data.addElement(sample);
		String[] groups = sample.getThreadGroups();
		Vector temp;
		for (int i = 0; i < groups.length; i++)
		{
			temp = (Vector) threadMap.get(groups[i]);
			if (temp == null)
			{
				temp = new Vector();
				threadMap.put(groups[i], temp);
			}
			temp.addElement(sample);
		}
		fireIntervalAdded(this, data.size() - 1, data.size() - 1);
	}


	/*************************
	 *  Description of the Method
	 *
	 *@param  sample  Description of Parameter
	 ************************/
	public void updateTestSample(TestSample sample)
	{
		String[] groups = sample.getThreadGroups();
		Set groupSet = new HashSet();
		for (int x = 0; x < groups.length; groupSet.add(groups[x++]))
		{
			;
		}
		Vector temp;
		int index = data.indexOf(sample);
		this.fireContentsChanged(this, index, index);
		for (int i = 0; i < groups.length; i++)
		{
			temp = (Vector) threadMap.get(groups[i]);
			if (temp == null)
			{
				temp = new Vector();
				threadMap.put(groups[i], temp);

			}
		}
		for (Iterator x = threadMap.keySet().iterator(); x.hasNext(); )
		{
			String name = (String) x.next();
			if (groupSet.contains(name))
			{
				temp = (Vector) threadMap.get(name);
				if (!temp.contains(sample))
				{
					temp.addElement(sample);
				}
			}
			else
			{
				temp = (Vector) threadMap.get(name);
				if (temp != null)
				{
					temp.remove(sample);
				}
			}
		}
	}


	/*************************
	 *  Description of the Method
	 *
	 *@param  indices  Description of Parameter
	 ************************/
	public void removeTestSamples(int[] indices)
	{
		List removing = new LinkedList();
		for (int x = 0; x < indices.length; x++)
		{
			removing.add(data.get(indices[x]));
		}
		for (Iterator x = removing.iterator(); x.hasNext(); )
		{
			TestSample ts = (TestSample) x.next();
			data.remove(ts);
			String[] groups = ts.getThreadGroups();
			for (int y = 0; y < groups.length; y++)
			{
				Vector temp = (Vector) threadMap.get(groups[y]);
				temp.remove(ts);
			}
		}
		for (int x = 0; x < indices.length; x++)
		{
			fireIntervalRemoved(this, indices[x], indices[x]);
		}
	}


	/*************************
	 *  Description of the Method
	 *
	 *@param  index  Description of Parameter
	 ************************/
	public void removeTestSample(int index)
	{
		removeTestSamples(new int[]{index});
	}
}
