/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;


import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.threads.*;


/**
 * Allows the user to control the thread groups in the system.
 * This is responsible for adding thread groups, and setting
 * the number of threads per group.
 */
public class ThreadGroupPanel
    extends JPanel
    implements ActionListener, ChangeListener
{
    public static final String[] DEFAULT = new String[] {"Default"};
    private ArrayList group_names;

    private JComboBox groups;
    private JSlider thread_count;
    private CardLayout slider_layout;
    private JTextField addGroup;

    private ArrayList counts;
    private ThreadMonitor[] monitors;
    private Vector listeners;

    private int start_threads;

    public ThreadGroupPanel() {
	this(DEFAULT);
    }

    public ThreadGroupPanel(String[] names) {
	super();
	listeners = new Vector();
	int max_threads=JMeterUtils.getPropDefault("threads.max",20);
	start_threads=JMeterUtils.getPropDefault("threads.start",10);
	group_names = new ArrayList();
        for(int x = 0;x < names.length; group_names.add(names[x++]));

	counts = new ArrayList();
	for(int i=0;i<group_names.size();i++)
	    counts.add(new Integer(start_threads));

	setLayout(new GridBagLayout());
	GridBagConstraints gbc = new GridBagConstraints();

	groups = new JComboBox((String[])group_names.toArray(new String[0]));
	groups.setActionCommand("Change_Group");
	groups.setEditable(false);
	groups.addActionListener(this);
	addGroup = new JTextField("");
	addGroup.setActionCommand("Add_Group");
	addGroup.addActionListener(this);

	thread_count = new JSlider(1, max_threads, start_threads);
	thread_count.setMajorTickSpacing(3);
	thread_count.setMinorTickSpacing(1);
	thread_count.setPaintTicks(true);
	thread_count.setPaintLabels(true);
	thread_count.setSnapToTicks(true);

	thread_count.addChangeListener(this);

	JPanel ledPanel = new JPanel();
	ledPanel.setMinimumSize(new Dimension(max_threads*7,20));
	monitors = new ThreadMonitor[max_threads];

	for(int i=0;i<max_threads;i++) {
	    ledPanel.add(monitors[i]=new ThreadMonitor());
	    monitors[i].setCurrentThreadGroup(names[0]);
	    if (i<start_threads)
		monitors[i].setEnabled(true);
	}


	//setPreferredSize(new Dimension(400, 100));

        gbc.weightx = 1;
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.anchor=gbc.EAST;
	gbc.insets = new Insets(2,2,2,2);
	add(new JLabel("Add Group", JLabel.LEFT),gbc.clone());
	gbc.gridy=1;
	add(new JLabel("Thread Group", JLabel.LEFT), gbc.clone());
	gbc.gridheight=1;
	gbc.gridx=2;
	gbc.gridy=0;
	gbc.anchor=gbc.WEST;
	gbc.fill=gbc.HORIZONTAL;
	add(addGroup,gbc.clone());
	gbc.gridy=1;
	add(groups, gbc.clone());
	gbc.gridy=2;
	gbc.gridx=0;
	gbc.gridwidth=3;
        gbc.fill = gbc.HORIZONTAL;
	add(thread_count, gbc.clone());

	gbc.gridy=3;
	add(ledPanel, gbc.clone());
    }

    public int getThreadCount(String thread_group) {
	int index;
        index = group_names.indexOf(thread_group);
	if(index > -1)
	  return ((Integer)counts.get(index)).intValue();
	else return -1;
    }

    public void disable() {
	thread_count.setEnabled(false);
	addGroup.setEnabled(false);
    }

    public void enable() {
	thread_count.setEnabled(true);
	addGroup.setEnabled(true);
    }

    public ThreadMonitor[] getMonitors() {
	return monitors;
    }

    public String[] getThreadGroupNames()
    {
	String[] ret = new String[groups.getItemCount()];
	for(int x = 0;x < ret.length;x++)
	    ret[x] = (String)groups.getItemAt(x);
	return ret;
    }

    public void addThreadGroupListener(ThreadGroupListener l) {
	listeners.addElement(l);
    }

    public void removeThreadGroupListener(ThreadGroupListener l) {
	listeners.removeElement(l);
    }

    //public void setThreads(String thread_group, JMeterThread[] threads) {

    public void actionPerformed(ActionEvent e) {
	String actionCommand = e.getActionCommand();
	if(actionCommand.equals("Change_Group")) {
	    thread_count.setValue(((Integer)counts.get(groups.getSelectedIndex())).intValue());
	    fireThreadGroupSelected(new ThreadGroupEvent((String)groups.getSelectedItem()));
	    for(int i=0;i<monitors.length;i++)
		monitors[i].setCurrentThreadGroup((String)groups.getSelectedItem());
	}
	else if(actionCommand.equals("Add_Group"))
	    {
		JTextField temp = (JTextField)e.getSource();
		groups.addItem(temp.getText());
                group_names.add(temp.getText());
		fireThreadGroupAdded(new ThreadGroupEvent(temp.getText()));
		counts.add(new Integer(start_threads));
		temp.setText("");
	    }
    }

    public void stateChanged(ChangeEvent e) {
	int count = thread_count.getValue();
	counts.set(groups.getSelectedIndex(),new Integer(count));
	for(int i=0;i<monitors.length;i++)
	    if (i<count)
		monitors[i].setEnabled(true);
	    else
		monitors[i].setEnabled(false);
    }

    public ThreadMonitor[] getThreadMonitors() {
	return monitors;
    }

    protected void fireThreadGroupAdded(ThreadGroupEvent e) {
	for(int i=0;i<listeners.size();i++)
	    ((ThreadGroupListener)listeners.elementAt(i)).threadGroupAdded(e);
    }

    protected void fireThreadGroupRemoved(ThreadGroupEvent e) {
	for(int i=0;i<listeners.size();i++)
	    ((ThreadGroupListener)listeners.elementAt(i)).threadGroupRemoved(e);
    }

    protected void fireThreadGroupSelected(ThreadGroupEvent e) {
	for(int i=0;i<listeners.size();i++)
	    ((ThreadGroupListener)listeners.elementAt(i)).threadGroupSelected(e);
    }

    public static void main(String[] args) {
	JFrame mainframe = new JFrame();
	ThreadGroupPanel tgp =
	    new ThreadGroupPanel(new String[] {"one", "two", "three"});
	mainframe.setSize(400, 200);
	mainframe.getContentPane().add(tgp);
	mainframe.setVisible(true);
    }
}


