/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.gui;

import javax.swing.*;
import java.awt.*;

import org.apache.jmeter.samplers.*;

/**
 * Provides a way of visualizing changes in thread group activity.
 */
public class ThreadMonitor extends JComponent implements SampleListener {
    private boolean active;
    private boolean enabled;
    private String threadGroup;
    private static final Dimension SIZE = new Dimension(5, 5);

    public ThreadMonitor() {
	this(false, false);
    }

    public ThreadMonitor(boolean active) {
	this(active, true);
    }

    public ThreadMonitor(boolean active, boolean enabled) {
	this.active=active;
	this.enabled=enabled;
	setSize(7, 7);
    }

    public void setCurrentThreadGroup(String threadGroup) {
	this.threadGroup=threadGroup;
    }

    public String getCurrentThreadGroup(String threadGroup) {
	return threadGroup;
    }

    public Dimension getPreferredSize() {
	return SIZE;
    }

    public void setActive(boolean active) {
	this.active=active;
	repaint();
    }

    public void setEnabled(boolean enabled) {
	this.enabled=enabled;
	repaint();
    }

    public boolean isActive() {
	return active;
    }

    public boolean isEnabled() {
	return enabled;
    }

    public void paint(Graphics g) {
	if (enabled) {
	    if (active)
		g.setColor(Color.green);
	    else
		g.setColor(Color.red);
	}
	else
	    g.setColor(new Color(128, 128, 128));

	g.fillRect(0, 0, getWidth(), getHeight());
    }

    public void sampleStarted(SampleEvent e) {
	if (threadGroup.equals(e.getThreadGroup())) {
	    active=true;
	    repaint();
	}
    }

    public void sampleOccurred(SampleEvent e) {
    }

    public void sampleStopped(SampleEvent e) {
	if (threadGroup.equals(e.getThreadGroup())) {
	    active=false;
	    repaint();
	}
    }
}
