package org.apache.jmeter.junit.protocol.http.config;

import junit.framework.TestCase;

import org.apache.jmeter.protocol.http.config.UrlConfig;
import org.apache.jmeter.config.Arguments;

/**
 * Title:        Apache JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache Foundation
 * @author Michael Stover
 * @version 1.0
 */

public class UrlConfigTest extends TestCase
{
	UrlConfig config;
	UrlConfig defaultConfig;
	UrlConfig partialConfig;

	public UrlConfigTest(String name)
	{
		super(name);
	}

	protected void setUp()
	{
		Arguments args = new Arguments();
		args.addArgument("username","mstover");
		args.addArgument("password","pass");
		args.addArgument("action","login");
		config = new UrlConfig();
		config.setName("Full Config");
		config.putProperty(config.DOMAIN,"www.lazer.com");
		config.putProperty(config.PATH,"login.jsp");
		config.putProperty(config.METHOD,config.POST);
		config.putProperty(config.ARGUMENTS,args);

		defaultConfig = new UrlConfig();
		defaultConfig.setName("default");
		defaultConfig.putProperty(config.DOMAIN,"www.xerox.com");
		defaultConfig.putProperty(config.PATH,"default.html");

		partialConfig = new UrlConfig();
		partialConfig.putProperty(config.PATH,"main.jsp");
		partialConfig.putProperty(config.METHOD,config.GET);
	}

	public void testSimpleConfig()
	{
		assert(config.getName().equals("Full Config"));
		assertEquals(config.getProperty(config.DOMAIN),"www.lazer.com");
	}

	public void testOverRide()
	{
		assertNull(partialConfig.getProperty(config.DOMAIN));
		partialConfig.addConfigElement(defaultConfig);
		assertEquals(partialConfig.getProperty(config.DOMAIN),"www.xerox.com");
		assertEquals(partialConfig.getProperty(config.PATH),"main.jsp");
	}


	public static void main(String[] args)
	{
	}
}