/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.protocol.ftp.control;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.threads.*;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.control.*;
import org.apache.jmeter.protocol.ftp.sampler.*;
import org.apache.jmeter.protocol.ftp.thread.*;

/**
 *  FTPSamplerController - a controller that implements simple
 *   FTP sampler
 */

public class FTPSamplerController implements SamplerController, ThreadGroupListener {

    /**
     *  default thread groups, required by some interface, only here for compatability
     */
    private static String[] defaultThreadGroups = {"Group 1"};


    /**
     *  Propery name, required by some interface, only here for compatability
     */
    String propertyName = "property";

    /**
     *  text field input the various parameters
     */
    JTextField usernameField = new JTextField(30);
    JPasswordField passwordField = new JPasswordField(30);
    JTextField serverField = new JTextField(30);
    JTextField fileField = new JTextField(30);

    public FTPSamplerController() {

    }



    public String[] getDefaultThreadGroups() {

	return defaultThreadGroups;

    }



    public void setTestSamples(String[] ts) {

        //System.out.println("set testsamples");

    }



    public void start() {
        usernameField.setEnabled(false);
	passwordField.setEnabled(false);
	serverField.setEnabled(false);
	fileField.setEnabled(false);
    }



    public void stop() {
        usernameField.setEnabled(true);
	passwordField.setEnabled(true);
	serverField.setEnabled(true);
	fileField.setEnabled(true);
    }



    /*****

     * Thread Group Listener Interface

     *****/

    public void threadGroupAdded(ThreadGroupEvent e) {

    }



    public void threadGroupRemoved(ThreadGroupEvent e) {

    }



    public void threadGroupSelected(ThreadGroupEvent e) {
    }



    /****

     * End ThreadGroupListener

     ****/



    public JMeterThread[] getSampleThreads(String threadGroup, int count) {
	FTPSampler s = new FTPSampler(usernameField.getText(), new String(passwordField.getPassword()), serverField.getText());

	JMeterThread[] result = new JMeterThread[count];

	for (int i=0;i<result.length;i++) {

	    result[i]=new FTPJMeterThread(s, fileField.getText());

	}

	return result;
    }


    public void addTestSample(TestSample testSample)

    {

        //System.out.println("add testsamples");

    }



    public Container getControlPanel() {

	JPanel panel = new JPanel();
	GridBagLayout g = new GridBagLayout();

	panel.setLayout(g);


	GridBagConstraints c = new GridBagConstraints();

	c.fill = c.BOTH;

	c.insets = new Insets(3, 3, 3, 3);

	c.gridx = 1;

	c.gridy = 1;

	c.gridwidth = 1;

	c.gridheight = 1;

	c.weightx = 0.0;

	c.weighty = 0.0;


	JLabel usernameLabel = new JLabel("Username:");
	c.gridx = 1;

	c.gridy = 2;

	g.setConstraints(usernameLabel, c);

	panel.add(usernameLabel);

	c.gridx = 2;

	c.gridy = 2;

	g.setConstraints(usernameField, c);

	panel.add(usernameField);

	JLabel passwordLabel = new JLabel("Password:");
	c.gridx = 1;

	c.gridy = 3;

	g.setConstraints(passwordLabel, c);

	panel.add(passwordLabel);

	c.gridx = 2;

	c.gridy = 3;

	g.setConstraints(passwordField, c);

	panel.add(passwordField);


	JLabel serverLabel = new JLabel("Server String:");
	c.gridx = 1;

	c.gridy = 4;

	g.setConstraints(serverLabel, c);

	panel.add(serverLabel);

	c.gridx = 2;

	c.gridy = 4;

	g.setConstraints(serverField, c);

	panel.add(serverField);

	JLabel fileLabel = new JLabel("File:");
	c.gridx = 1;

	c.gridy = 5;

	g.setConstraints(fileLabel, c);

	panel.add(fileLabel);

	c.gridx = 2;

	c.gridy = 5;

	g.setConstraints(fileField, c);

	panel.add(fileField);

	return panel;

    }



    public String getName()

    {

	return "FTP Base Sampler";

    }



    public String getPropName()

    {

      return propertyName;

    }



    public void setPropName(String p) {
        propertyName = p;

    }



    public String toString() {

	return getName();

    }



}

