package org.apache.jmeter.protocol.http.config;

import java.util.*;

import org.apache.jmeter.config.*;

/*************************
 *  Title: Apache JMeter
 *  Description:
 *  Copyright: Copyright (c) 2000
 *  Company: Apache Foundation
 *
 *@author     Michael Stover
 *@created    $Date: 2000/12/22 01:41:48 $
 *@version    $Revision: 1.1 $
 ************************/

public class UrlConfig implements ConfigElement
{
	public static final String DOMAIN = "domain";
	public static final String PATH = "path";
	public static final String METHOD = "method";
	public static final String ARGUMENTS = "arguments";
	public static final String POST = "post";
	public static final String GET = "get";

	private String name;

	private Map properties;

	/*************************
	 *  Constructor for the UrlConfig object
	 ************************/
	public UrlConfig()
	{
		properties = new HashMap();
	}

	/**
	 * Sets the name of the config element.
	 */
	public void setName(String name)
	{
		this.name = name;
	}

	/**
	 * Gets the name of the config element.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Gets a property of the config element.
	 */
	public Object getProperty(String key)
	{
		return properties.get(key);
	}

	/**
	 * Sets a property of the config element.
	 */
	public void putProperty(String key,Object value)
	{
		properties.put(key,value);
	}

	/**
	 * Get the names of all valid properties for this config element.
	 */
	public Collection getPropertyNames()
	{
		return properties.keySet();
	}

	/**
	 * This allows config elements to combine and give a "layered" effect.  for
	 * example, say there are two HTTPConfigElements, which have properties for
	 * domain, path, method, and parameters.  If element A has everything filled
	 * in, but null for domain, and element B is added, which has only domain filled
	 * in, then after adding B to A, A will have the domain from B.  If A already
	 * had a domain, then the correct behavior is for A to ignore the addition of
	 * element B.
	 */
	public void addConfigElement(ConfigElement config)
	{
		if(config instanceof UrlConfig)
		{
			for(Iterator it = config.getPropertyNames().iterator();it.hasNext();)
			{
				String propName = (String)it.next();
				if(!contains(propName))
				{
					properties.put(propName,config.getProperty(propName));
				}
			}
		}
	}

	private boolean contains(String key)
	{
		Object value = properties.get(key);
		if(value == null || value.toString().equals(""))
		{
			return false;
		}
		else
		{
			return true;
		}
	}
}
