/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.protocol.http.control;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.*;

/**
 * This class provides an interface to the netscape cookies file to
 * pass cookies along with a request.
 *
 * @author  <a href="mailto:sdowd@arcmail.com">Sean Dowd</a>
 * @version $Revision: 1.1 $ $Date: 2000/11/19 23:31:38 $
 */
public class CookieManager extends AbstractTableModel implements Cloneable {
    /**
     * A vector of Cookies managed by this class.
     * @associates <{org.apache.jmeter.controllers.Cookie}>
     */
    private Vector cookies = new Vector();
    private static SimpleDateFormat dateFormat = new SimpleDateFormat("EEEE, dd-MMM-yyyy HH:mm:ss zzz");

    private final static int columnCount = 6;
    private final static String[] columnNames = {"Name","Value","Domain","Path","Secure","Expiration"};

    public CookieManager () {
    }

    public Object clone()
    {
        CookieManager ck = new CookieManager();
        for(int x = 0;x < cookies.size();x++)
          ck.add((Cookie)cookies.get(x));
        return ck;
    }

    /** required by table model interface */
   public int getRowCount() {
       return cookies.size();
   }
   /** required by table model interface */
   public int getColumnCount() {
       return columnCount;
   }
   /** required by table model interface */
   public String getColumnName(int column) {
       return columnNames[column];
   }
   /** required by table model interface */
   public Class getColumnClass(int column) {
       return columnNames[column].getClass();
   }
   /** required by table model interface */
   public Object getValueAt(int row, int column) {
	Cookie cook = (Cookie) cookies.elementAt(row);
        if (column == 0) {
            return cook.getName();
        } else if (column == 1) {
            return cook.getValue();
        } else if (column == 2) {
            return cook.getDomain();
        } else if (column == 3) {
            return cook.getPath();
        } else if (column == 4) {
            return new Boolean(cook.getSecure());
        } else if (column == 5) {
            return new Long(cook.getExpires());
        }
	return null;
   }

    /** save the cookie data to a file */
    public void save(String authFile) throws IOException {
       File file = new File(authFile);
        if (!file.isAbsolute()) file = new File(System.getProperty("user.dir") + File.separator + authFile);
        PrintWriter writer = new PrintWriter(new FileWriter(file));
	writer.println("# JMeter generated Cookie file");
	for (int i = 0; i < cookies.size(); i++) {
		Cookie cook = (Cookie) cookies.elementAt(i);
		writer.println(cook.toString());
	}
	writer.flush();
	writer.close();
    }

    /** add cookie data from a file */
    public void addFile (String cookieFile) throws IOException {
	File file = new File(cookieFile);
        if (!file.isAbsolute()) file = new File(System.getProperty("user.dir") + File.separator + cookieFile);
	BufferedReader reader = null;
	if( file.canRead() ) {
        	reader = new BufferedReader(new FileReader(file));
	} else {
		JOptionPane.showMessageDialog(null, "The file you specified cannot be read.",
					"Information", JOptionPane.INFORMATION_MESSAGE);
		return;
	}

        String line;
        while ((line = reader.readLine()) != null) {
            try {
                if (line.startsWith("#") || line.trim().length() == 0) continue;
                String[] st = split(line, "\t"," ");
                int domain = 0;
                int foo = 1;
                int path = 2;
                if(st[path].equals(" "))
                  st[path] = "/";
                boolean secure = new Boolean(st[3]).booleanValue();
                long expires = new Long(st[4]).longValue();
                int name = 5;
                int value = 6;
                Cookie cookie = new Cookie(st[name], st[value], st[domain],
                        st[path], secure, expires);
                cookies.addElement(cookie);
            } catch (Exception e) {
                throw new IOException("Error parsing cookie line\n\t'" + line + "'\n\t" + e);
            }
        }
        reader.close();
        fireTableChanged(new TableModelEvent(this));
    }

    /** add a cookie */
    public void add(Cookie c) {
        cookies.addElement(c);
        fireTableChanged(new TableModelEvent(this));
    }
    /** remove a cookie */
    public void remove(int index) {
        cookies.removeElementAt(index);
        fireTableChanged(new TableModelEvent(this));
    }

   /** return the number cookies */
   public int size() {
       return cookies.size();
   }

   /** return the cookie at index i */
   public Cookie get(int i) {
       return (Cookie) cookies.elementAt(i);
   }

   public String getCookieHeaderForURL(URL url) {
	if (! url.getProtocol().toUpperCase().trim().equals("HTTP")
	    &&
	    ! url.getProtocol().toUpperCase().trim().equals("HTTPS")) return null;

        StringBuffer header = new StringBuffer();
        for (Enumeration enum = cookies.elements(); enum.hasMoreElements();) {
            Cookie cookie = (Cookie) enum.nextElement();
            if (url.getHost().endsWith(cookie.getDomain()) &&
                    url.getFile().startsWith(cookie.getPath()) &&
                    (System.currentTimeMillis() / 1000) <= cookie.getExpires()) {
                if (header.length() > 0) {
                    header.append("; ");
                }
                header.append(cookie.getName()).append("=").append(cookie.getValue());
            }
        }

        if (header.length() != 0) {
            return header.toString();
        } else {
            return null;
        }
    }

    public void addCookieFromHeader(String cookieHeader, URL url) {
        StringTokenizer st = new StringTokenizer(cookieHeader, ";");
        String nvp;

        // first n=v is name=value
        nvp = st.nextToken();
        StringTokenizer nvpTokenizer = new StringTokenizer(nvp, "=");
        String name = nvpTokenizer.nextToken();
        String value = nvpTokenizer.nextToken();
        String domain = url.getHost();
        String path = url.getFile();

        Cookie newCookie = new Cookie(name, value, domain, path, false,
                System.currentTimeMillis() + 1000 * 60 * 60 * 24);
        // check the rest of the headers
        while (st.hasMoreTokens()) {
            nvp = st.nextToken();
            nvp = nvp.trim();
            nvpTokenizer = new StringTokenizer(nvp, "=");
            String key = nvpTokenizer.nextToken();
            if (key.equalsIgnoreCase("expires")) {
                try {
                    String expires = nvpTokenizer.nextToken();
                    Date date = dateFormat.parse(expires);
                    newCookie.setExpires(date.getTime());
                } catch (ParseException pe) {}
            } else if (key.equalsIgnoreCase("domain")) {
                newCookie.setDomain(nvpTokenizer.nextToken());
            } else if (key.equalsIgnoreCase("path")) {
                newCookie.setPath(nvpTokenizer.nextToken());
            } else if (key.equalsIgnoreCase("secure")) {
                newCookie.setSecure(true);
            }
        }

        Vector removeIndices = new Vector();
        for (int i = cookies.size() - 1; i >= 0; i--) {
            Cookie cookie = (Cookie) cookies.elementAt(i);
            if (cookie == null)
                continue;
            if (cookie.getPath().equals(newCookie.getPath()) &&
                    cookie.getDomain().equals(newCookie.getDomain()) &&
                    cookie.getName().equals(newCookie.getName())) {
                removeIndices.addElement(new Integer(i));
            }
        }

        for (Enumeration e = removeIndices.elements(); e.hasMoreElements();) {
            int index = ((Integer) e.nextElement()).intValue();
            cookies.removeElementAt(index);
        }

        if (newCookie.getExpires() >= System.currentTimeMillis())
            cookies.addElement(newCookie);
    }

    public void removeCookieNamed(String name) {
        Vector removeIndices = new Vector();
        for (int i = cookies.size() - 1; i > 0; i--) {
            Cookie cookie = (Cookie) cookies.elementAt(i);
            if (cookie == null)
                continue;
            if (cookie.getName().equals(name))
                removeIndices.addElement(new Integer(i));
        }

        for (Enumeration e = removeIndices.elements(); e.hasMoreElements();) {
            cookies.removeElementAt(((Integer) e.nextElement()).intValue());
        }

    }

    /******************************************************
     * Takes a String and a tokenizer character, and returns
     a new array of strings of the string split by the tokenizer
     character.
     @param splittee String to be split
     @param splitChar Character to split the string on
     @param def Default value to place between two split chars that have
     nothing between them
     @return Array of all the tokens.
    ******************************************************/
    public String[] split(String splittee,String splitChar,String def)
    {
  	if(splittee == null || splitChar == null)
	    return new String[0];
	StringTokenizer tokens;
	String temp;
	int spot;
	while((spot=splittee.indexOf(splitChar+splitChar))!=-1)
	    splittee=splittee.substring(0,spot+splitChar.length())+def+
		splittee.substring(spot+1*splitChar.length(),
				   splittee.length());
	Vector returns=new Vector();
	tokens=new StringTokenizer(splittee,splitChar);
	while(tokens.hasMoreTokens()) {
	    temp=(String)tokens.nextToken();
	    returns.addElement(temp);
	}
	String[] values=new String[returns.size()];
	returns.copyInto(values);
	return values;
    }
}
