package org.apache.jmeter.protocol.http.control;

import org.apache.jmeter.samplers.*;
import org.apache.jmeter.control.*;
import org.apache.jmeter.threads.*;
import org.apache.jmeter.protocol.http.gui.*;
import org.apache.jmeter.protocol.http.thread.HTTPJMeterThread;
import org.apache.jmeter.protocol.http.sampler.*;
import java.net.URL;

public class SimpleHTTPController implements SamplerController {
    String url;
    String propName;
    String method;
    SimpleHTTPControllerGUI gui;

    private static final String[] DEFAULT_GROUPS = new String[] {"Group 1"};

    public SimpleHTTPController() {
    }

    public void setURL(String url) {
    	this.url=url;
    }

    public String getURL()
    {
      return this.url;
    }

    /**
     * Tells the sampler to begin collecting samples in its own special way.
     */
    public void start() {
	// disable gui
    }

    /**
     * Tells the sampler to cease collecting samples in its own special way.
     */
    public void stop() {
	// enable gui
    }

    /**
     * Returns a container which knows how to set various parameters of
     * the sampler.
     */
    public java.awt.Container getControlPanel() {
	    if(gui == null)
	      gui = new SimpleHTTPControllerGUI(this);
      else gui.setURL(url);
	    return gui;
    }

    /**
     * Tells what the sampler likes to be called.
     */
    public String getName() {
	return "Simple HTTP Tester";
    }

    public String toString()
    {
      return getName();
    }

    /**
     * Tells what the sampler is called in the properties file
     */
    public String getPropName() {
	return propName;
    }

    /**
     * Sets the samplers name in the properties file
     */
    public void setPropName(String propName) {
	this.propName=propName;
    }

    /**
     * Tells which thread groups the sampler wants to belong to,
     * unless told otherwise.
     */
    public String[] getDefaultThreadGroups() {
	return DEFAULT_GROUPS;
    }

    /**
     * Returns the actual sampler thread(s) that will be doing the work.
     * @associates <{org.apache.jmeter.controllers.JMeterThread}>
     */
    public JMeterThread[] getSampleThreads(String threadGroup, int count) {
        HTTPBaseTestSample hbt = new HTTPBaseTestSample();
	hbt.setUrl(url);
	hbt.setOnce(false);
	hbt.setMethod("get");
	hbt.setThreadGroups(new String[] {threadGroup});

	TestSample[] samples = new TestSample[] {hbt};
	if (samples != null && samples.length > 0) {
	    JMeterThread[] result = new JMeterThread[count];
	    for (int i=0;i<result.length;i++)
      {
	  result[i]=new HTTPJMeterThread(new HTTPSampler(new CookieManager(),
            new AuthManager()),samples,threadGroup);
      }
	    return result;
	}
	return null;
    }

    /**
     * Sets the list of TestSamples that this SamplerController will use.
     */
    public void setTestSamples(String[] testSamples) {
    }

    /*****
     * Thread Group Listener Interface
     *****/
    public void threadGroupAdded(ThreadGroupEvent e) {}
    public void threadGroupRemoved(ThreadGroupEvent e) {}
    public void threadGroupSelected(ThreadGroupEvent e) {}
}

