package org.apache.jmeter.protocol.http.gui;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

import org.apache.jmeter.samplers.*;
import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.protocol.http.control.*;

/**
 * Handles input for determining if authentication services
 *  are required for a Sampler.  It also understands how to
 * get AuthManagers for the files that the user selects.
 */
public class AuthPanel extends JPanel implements ActionListener {
    /**
     * Allows the user enter the data with a GUI
     * interface
     */
    JButton specifyButton;
    JCheckBox check;

    /**
     * The authentication manager for this object
     */
    AuthManager manager = new AuthManager();

    /**
     * A table to show the authentication information
     */
    JTable authTable;

    /**
     * Default Constructor
     **/
    public AuthPanel() {
	EtchedBorder etched = new EtchedBorder();
	setBorder(new TitledBorder(etched, "Authorization"));
        setLayout(new FlowLayout(FlowLayout.LEFT));
        check = new JCheckBox("Enable");
        check.addActionListener(this);
        add(check);
	specifyButton = new JButton("Config");
	specifyButton.setEnabled(false);
	specifyButton.setActionCommand("edit");
	specifyButton.addActionListener(this);
	add(specifyButton);
    }

    /**
     * Disable the panel
     **/
    public void disable() {
	check.setEnabled(false);
	specifyButton.setEnabled(false);
    }

    /**
     * Enable the panel
     **/
    public void enable() {
	check.setEnabled(true);
        if(check.isSelected()) {
            specifyButton.setEnabled(true);
        } else {
            specifyButton.setEnabled(false);
        }
    }

    /**
     * Handles the action events:
     * <DL>
     * <DT>"authorization"
     * <DD>Toggles the activation ability on or off.
     * <DT>"browse"
     * <DD>Shows the file chooser for them to
     *  pick (another) authorization file.
     * </DL>
     */
    public void actionPerformed(ActionEvent e) {
	if (e.getActionCommand().equals("edit")) {
	    showEditDialog();
	} else {
	    if(check.isSelected()) {
                specifyButton.setEnabled(true);
	    } else {
                specifyButton.setEnabled(false);
	    }
	}
    }

    /**
     * Retrieves the AuthManager for the currently selected file.
     *@returns an AuthManager for the current file, or null if:
     * a file is not selected or that file cannot be read.
     */
    public AuthManager getAuthMgr() {
	return check.isSelected() ? manager : null;
    }

    /**
     * Shows the main authentication panel for this object
     */
    public void showEditDialog() {
		final JDialog entryDialog = new JDialog();
		entryDialog.setSize(400,500);
                authTable = new JTable(manager);
                authTable.setCellSelectionEnabled(false);
                authTable.setRowSelectionAllowed(true);
		authTable.setColumnSelectionAllowed(false);
                authTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

                JPanel main = new JPanel();
                main.setLayout(new BorderLayout());

		JScrollPane scroller = new JScrollPane(authTable);
                JTableHeader tableHeader = authTable.getTableHeader();
		scroller.setColumnHeaderView(tableHeader);
		main.add(scroller);

		JButton add = new JButton("Add");
		JButton edit = new JButton("Edit");
		JButton delete = new JButton("Delete");
		JButton close = new JButton("Close");
		JButton load = new JButton("Load");
		JButton save = new JButton("Save");
                Dimension d = delete.getPreferredSize();
                add.setPreferredSize(d);
                edit.setPreferredSize(d);
                close.setPreferredSize(d);
                load.setPreferredSize(d);
                save.setPreferredSize(d);

		add.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
                                final JDialog addDialog = new JDialog();
		                addDialog.setSize(300,150);

                                JPanel _main = new JPanel();
                                GridBagLayout g = new GridBagLayout();
                                _main.setLayout(g);
                                GridBagConstraints c = new GridBagConstraints();
                                c.fill = c.BOTH;
                                c.gridwidth = 1;
                                c.gridheight = 1;
                                c.gridx = 1;
                                c.gridy = 1;
                                JLabel urlLabel = new JLabel("Base URL:");
                                g.setConstraints(urlLabel, c);

                                _main.add(urlLabel);
                                final JTextField nameField = new JTextField(20);
                                c.gridx = 2;
                                c.gridy = 1;
                                g.setConstraints(nameField, c);
                                _main.add(nameField);
                                JLabel usernameLabel = new JLabel("Username:");
				c.gridx = 1;
                                c.gridy = 2;
                                g.setConstraints(usernameLabel, c);
                                _main.add(usernameLabel);
                                final JTextField usernameField = new JTextField(20);
				c.gridx = 2;
                                c.gridy = 2;
                                g.setConstraints(usernameField, c);
                                _main.add(usernameField);
                                JLabel passwordLabel = new JLabel("Password:");
				c.gridx = 1;
                                c.gridy = 3;
                                g.setConstraints(passwordLabel, c);
                                _main.add(passwordLabel);
                                final JPasswordField passwordField = new JPasswordField(20);
				c.gridx = 2;
                                c.gridy = 3;
                                g.setConstraints(passwordField, c);
                                _main.add(passwordField);
                                JButton ok = new JButton("Ok");
                                JButton cancel = new JButton("Cancel");
                                ok.setPreferredSize(cancel.getPreferredSize());
                                ok.addActionListener(new ActionListener() {
                                    public void actionPerformed(ActionEvent e) {
				        manager.set(-1, nameField.getText(), usernameField.getText(), new String(passwordField.getPassword()));
                                        addDialog.dispose();
                                    }
                                });
                                cancel.addActionListener(new ActionListener() {
                                    public void actionPerformed(ActionEvent e) {
                                        addDialog.dispose();
                                    }
                                });
                                c.fill = c.NONE;
                                c.anchor = c.EAST;
				c.gridx = 1;
                                c.gridy = 4;
                                g.setConstraints(ok, c);
                                _main.add(ok);
                                c.anchor = c.WEST;
				c.gridx = 2;
                                c.gridy = 4;
                                g.setConstraints(cancel, c);
				_main.add(cancel);
                                addDialog.getContentPane().add(_main);
				addDialog.show();
			}
		});
		edit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final int row = authTable.getSelectedRow();
                                if (row >= 0) {
                                    Authorization auth = manager.get(row);
                                    final JDialog addDialog = new JDialog();
                                    addDialog.setSize(300,150);

                                    JPanel _main = new JPanel();
                                    GridBagLayout g = new GridBagLayout();

                                    _main.setLayout(g);
                                    GridBagConstraints c = new GridBagConstraints();
                                    c.fill = c.BOTH;
                                    c.gridwidth = 1;
                                    c.gridheight = 1;
                                    c.gridx = 1;
                                    c.gridy = 1;
                                    JLabel urlLabel = new JLabel("Base URL:");
                                    g.setConstraints(urlLabel, c);

                                    _main.add(urlLabel);
                                    final JTextField nameField = new JTextField(auth.getURL(), 20);
                                    c.gridx = 2;
                                    c.gridy = 1;
                                    g.setConstraints(nameField, c);
                                    _main.add(nameField);
                                    JLabel usernameLabel = new JLabel("Username:");
                                    c.gridx = 1;
                                    c.gridy = 2;
                                    g.setConstraints(usernameLabel, c);
                                    _main.add(usernameLabel);
                                    final JTextField usernameField = new JTextField(auth.getUser(), 20);
                                    c.gridx = 2;
                                    c.gridy = 2;
                                    g.setConstraints(usernameField, c);
                                    _main.add(usernameField);
                                    JLabel passwordLabel = new JLabel("Password:");
                                    c.gridx = 1;
                                    c.gridy = 3;
                                    g.setConstraints(passwordLabel, c);
                                    _main.add(passwordLabel);
                                    final JPasswordField passwordField = new JPasswordField(auth.getPass(), 20);
                                    c.gridx = 2;
                                    c.gridy = 3;
                                    g.setConstraints(passwordField, c);
                                    _main.add(passwordField);
                                    JButton ok = new JButton("Ok");
                                    JButton cancel = new JButton("Cancel");
                                    ok.setPreferredSize(cancel.getPreferredSize());
                                    ok.addActionListener(new ActionListener() {
                                        public void actionPerformed(ActionEvent e) {
				            manager.set(row, nameField.getText(), usernameField.getText(), new String(passwordField.getPassword()));
                                            addDialog.dispose();
                                        }
                                    });
                                    cancel.addActionListener(new ActionListener() {
                                        public void actionPerformed(ActionEvent e) {
                                            addDialog.dispose();
                                        }
                                    });
                                    c.fill = c.NONE;
                                    c.anchor = c.EAST;
                                    c.gridx = 1;
                                    c.gridy = 4;
                                    g.setConstraints(ok, c);
                                    _main.add(ok);
                                    c.anchor = c.WEST;
                                    c.gridx = 2;
                                    c.gridy = 4;
                                    g.setConstraints(cancel, c);
                                    _main.add(cancel);
                                    addDialog.getContentPane().add(_main);
                                    addDialog.show();
                                }
			}
		});
		delete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int row = authTable.getSelectedRow();
				if (row >= 0) {
					manager.remove(row);
				}
			}
		});
		close.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				entryDialog.dispose();
			}
		});
		load.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					File tmp = JMeterUtils.promptToOpenFile()
							.getSelectedFile();
					if (tmp != null) {
						manager.addFile(tmp.getAbsolutePath());
					}
				} catch (IOException ex) {
					ex.printStackTrace();
				}
			}
		});
		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					File tmp = JMeterUtils.promptToOpenFile()
							.getSelectedFile();
					if (tmp != null) {
						manager.save(tmp.getAbsolutePath());
					}
				} catch (IOException ex) {
					ex.printStackTrace();
				}
			}
		});

		JPanel panel = new JPanel();
                GridBagLayout g = new GridBagLayout();
                panel.setLayout(g);
                GridBagConstraints c = new GridBagConstraints();
                c.fill = c.NONE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.gridx = 1;
		c.gridy = 1;
		g.setConstraints(add, c);
		panel.add(add);
		c.gridx = 2;
		c.gridy = 1;
		g.setConstraints(edit, c);
		panel.add(edit);
		c.gridx = 3;
		c.gridy = 1;
		g.setConstraints(delete, c);
		panel.add(delete);
		c.gridx = 1;
		c.gridy = 2;
		g.setConstraints(close, c);
		panel.add(close);
		c.gridx = 2;
		c.gridy = 2;
		g.setConstraints(load, c);
		panel.add(load);
		c.gridx = 3;
		c.gridy = 2;
		g.setConstraints(save, c);
		panel.add(save);
                main.add(panel, "South");
		entryDialog.getContentPane().add(main);
		entryDialog.show();
    }
}
