/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.protocol.jdbc.thread;
import java.util.Vector;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.threads.*;

import org.apache.jmeter.protocol.jdbc.sampler.*;
/**
 * JDBCJMeterThread - thread for JDBC Sampler
 */
public class JDBCJMeterThread implements JMeterThread {
    // Sampler for this thread
    JDBCSampler sampler;
    // flag to determine stopping point
    boolean running = true;
    Vector listeners = new Vector();
    Timer timer;
    // SQL query to perform on database
    String query;
    public JDBCJMeterThread(JDBCSampler sampler, String query) {
	this.sampler = sampler;
	this.query = query;
    }

    public void stopSampling() {
	running = false;
    }

    public String getThreadGroupName() {
        return "Group 1";
    }

    public void addSampleListener(SampleListener l) {
	listeners.addElement(l);
    }

    public void removeSampleListener(SampleListener l) {
	listeners.removeElement(l);
    }

    public void setTimer(org.apache.jmeter.timers.Timer timer) {
	this.timer = timer;
    }

    public void run() {
	SampleResult res;
	timer.set();
	while(running) {
	    long start = System.currentTimeMillis();
	    fireSampleStartedEvent();
	    Entry e = new Entry();
	    e.putValue(JDBCSampler.QUERY, query);
	    res = sampler.sample(e);
	    fireSampleStoppedEvent();
	    fireSampleOccurredEvent(res);
	    long end = System.currentTimeMillis();
            try {
		long sleeptime = timer.delay() - (end - start);
		if (sleeptime > 0) {
			Thread.currentThread().sleep(sleeptime);
		}
	    } catch (InterruptedException ex) {
	    }
            Thread.currentThread().yield();
	}
    }
    protected void fireSampleOccurredEvent(SampleResult result) {
	SampleEvent e = new SampleEvent(result,getThreadGroupName());
	synchronized(listeners) {
	    for(int i=0;i<listeners.size();i++) {
		SampleListener l = (SampleListener)listeners.elementAt(i);
		l.sampleOccurred(e);
	    }
	}
    }

    protected void fireSampleStartedEvent() {
      synchronized(listeners) {
	  for(int i=0;i<listeners.size();i++) {
	      SampleListener l = (SampleListener)listeners.elementAt(i);
	      l.sampleStarted(new SampleEvent(null,getThreadGroupName()));
	  }
      }
    }

    protected void fireSampleStoppedEvent() {
	synchronized(listeners) {
	    for(int i=0;i<listeners.size();i++) {
		SampleListener l = (SampleListener)listeners.elementAt(i);
		l.sampleStopped(new SampleEvent(null,getThreadGroupName()));
	    }
	}
    }
}



