/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.jmeter.samplers;

import java.util.*;
import java.io.Serializable;

/**
This is a nice packaging for the various information returned from taking a sample of an entry.
**/
public class SampleResult implements Serializable
{

  Map map;
  long time;

  public SampleResult()
  {
    map = new HashMap();
    time = 0;
  }

    /**
     * Set the time this sample took to occur.
     **/
  public void setTime(long t)
  {
    time = t;
  }

    /**
     * Get the time it took this sample to occur.
     **/
  public long getTime()
  {
    return time;
  }
  /**
  All keys in this interface must be Strings.
  **/
  public Object getValue(String key)
  {
    Object value = map.get(key);
    return value;
  }

  public void putValue(String key,Object value)
  {
    map.put(key,value);
  }

  /**
  Get a ready made iterator to go through the names
  **/
  public java.util.Iterator nameIterator()
  {
    return map.keySet().iterator();
  }

  /**
    Remove a key-value pair.
  **/
  public void remove(String key)
  {
    map.remove(key);
  }

  /**
  Get an array of all the names
  **/
  public String[] getNames()
  {
    return (String[])map.keySet().toArray(new String[0]);
  }

  /**
  Generates an iterator for all the values stored under one key.
  **/
  public java.util.Iterator valueIterator(String key)
  {
    Object value = null;
    Object[] values = null;
    try{
      values = (Object[])map.get(key);
    }catch(Exception e){value = map.get(key);}
    if(value == null)
    {
      if(values == null)
      {
        List list = new LinkedList();
        return list.iterator();
      }
      else
      {
        List list = new LinkedList();
        for(int x = 0;x < values.length;list.add(values[x++]));
        return list.iterator();
      }
    }
    else if(value instanceof Collection)
    {
      Collection col = (Collection)value;
      return col.iterator();
    }
    else
    {
      List list = new LinkedList();
      list.add(value);
      return list.iterator();
    }
  }

  /**
    Get the value for the key as a String object
  **/
  public String toString(String key)
  {
    Object value = map.get(key);
    if(value == null)
      value = new String("");
    return value.toString();
  }

}
