/*
 * Copyright (c) 1998-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.visualizers;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

import org.apache.jmeter.samplers.*;

/**
 * Allows the tester to view the textual response from sampling
 * an Entry.  This also allows to "single step through" the sampling
 * process via a nice "Continue" button.
 *
 * @author  $Author: mstover $
 * @version $Revision: 1.7 $ $Date: 2000/09/12 02:27:27 $
 */
public class ViewResultsVisualizer extends JPanel implements Visualizer, ActionListener {



    private JTextArea display;
    private JButton continu;
    private JTextField label;

    Object sync = new Object();

    public ViewResultsVisualizer() {
        super();
        label = new JTextField();
        display = new JTextArea();
        continu = new JButton("Continue");
        continu.addActionListener(this);
        setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = gbc.gridy = 0;
        gbc.fill = gbc.BOTH;
        gbc.weightx = 1;
        gbc.weighty = 0;
        this.add(label,gbc.clone());

        gbc.gridy++;
        gbc.weighty = 1;
        this.add(new JScrollPane(display),gbc.clone());

        gbc.gridy++;
        gbc.fill = gbc.NONE;
        gbc.weightx = gbc.weighty = 0;
        gbc.anchor = gbc.CENTER;
        this.add(continu,gbc.clone());
    }



    public synchronized void add(SampleResult sampleResult) {
        label.setText((String)sampleResult.getValue(Sampler.SAMPLE_LABEL));
        display.setText(sampleResult.getValue(Sampler.TEXT_RESPONSE).toString());
        try {
          wait();
        }catch(InterruptedException e){}
    }

    public synchronized void clear()
    {
      label.setText("");
      display.setText("");
    }

    public void actionPerformed(ActionEvent e)
    {
      synchronized(this)
      {
        this.notify();
      }
    }

    public String toString() {
        return "Show text response from sample";
    }

    public JPanel getControlPanel() {
      synchronized(this)
      {
        this.notify();
      }
	return this;
    }
}
